/*
 * Copyright 2019 Confluent Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package io.confluent.connect.secretregistry.rbac.config.provider;

import java.util.Map;
import java.util.Properties;
import java.util.Set;
import java.util.stream.Collectors;

import io.confluent.connect.secretregistry.ConnectSecretRegistryExtension;
import io.confluent.kafka.secretregistry.client.rest.entities.Secret;
import io.confluent.kafka.secretregistry.rest.SecretRegistryConfig;
import io.confluent.kafka.secretregistry.storage.KafkaSecretRegistry;

import org.apache.kafka.common.config.ConfigData;
import org.apache.kafka.common.config.provider.ConfigProvider;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


/**
 * An implementation of {@link ConfigProvider} that obtains secrets from the secret registry.
 */
public class InternalSecretConfigProvider implements ConfigProvider {

  private static final Logger log = LoggerFactory.getLogger(InternalSecretConfigProvider.class);

  private static String PARAM_PREFIX = "param.";

  private static KafkaSecretRegistry secretRegistry;

  private static synchronized void initSecretRegistry(Map<String, ?> configs) {
    try {
      if (secretRegistry == null) {
        Properties props = new Properties();
        for (Map.Entry<String, ?> entry : configs.entrySet()) {
          String key = entry.getKey();
          if (key.startsWith(PARAM_PREFIX)) {
            key = key.substring(PARAM_PREFIX.length());
          }
          props.put(key, entry.getValue());
        }
        SecretRegistryConfig config = new SecretRegistryConfig(props);
        secretRegistry = new KafkaSecretRegistry(config);
        secretRegistry.initStore();
        ConnectSecretRegistryExtension.configureSecretRegistry(secretRegistry);
        log.info("Done initializing InternalSecretConfigProvider");
      }
    } catch (Exception e) {
      secretRegistry = null;
      log.warn("Error initializing InternalSecretConfigProvider", e);
    }
  }

  public void configure(Map<String, ?> configs) {
    initSecretRegistry(configs);
  }

  /**
   * Retrieves the data from the secret registry at the given path.
   *
   * @param path the path where the data resides
   * @return the configuration data
   */
  public ConfigData get(String path) {
    try {
      Map<String, String> data = secretRegistry.getAllLatest(path).stream()
          .collect(Collectors.toMap(Secret::getKey, Secret::getSecret));
      return new ConfigData(data);
    } catch (Exception e) {
      log.error("Could not obtain config data", e);
      throw new RuntimeException(e);
    }
  }

  /**
   * Retrieves the data with the given keys at the given path.
   *
   * @param path the path where the data resides
   * @param keys the keys whose values will be retrieved
   * @return the configuration data
   */
  public ConfigData get(String path, Set<String> keys) {
    try {
      Map<String, String> data = secretRegistry.getAllLatest(path).stream()
          .filter(s -> keys.contains(s.getKey()))
          .collect(Collectors.toMap(Secret::getKey, Secret::getSecret));
      return new ConfigData(data);
    } catch (Exception e) {
      log.error("Could not obtain config data", e);
      throw new RuntimeException(e);
    }
  }

  public void close() {
  }
}
